import warnings

warnings.warn(
    "wpilib.trajectory.constraint has moved to wpimath.trajectory.constraint",
    FutureWarning,
)


from wpimath.trajectory.constraint import (
    CentripetalAccelerationConstraint,
    DifferentialDriveKinematicsConstraint,
    DifferentialDriveVoltageConstraint,
    MecanumDriveKinematicsConstraint,
    SwerveDrive3KinematicsConstraint,
    SwerveDrive4KinematicsConstraint,
    SwerveDrive6KinematicsConstraint,
    TrajectoryConstraint,
)

__all__ = [
    "CentripetalAccelerationConstraint",
    "DifferentialDriveKinematicsConstraint",
    "DifferentialDriveVoltageConstraint",
    "MecanumDriveKinematicsConstraint",
    "SwerveDrive3KinematicsConstraint",
    "SwerveDrive4KinematicsConstraint",
    "SwerveDrive6KinematicsConstraint",
    "TrajectoryConstraint",
]



import warnings

warnings.warn("wpilib.trajectory has moved to wpimath.trajectory", FutureWarning)

from wpimath.trajectory import (
    Trajectory,
    TrajectoryConfig,
    TrajectoryGenerator,
    TrajectoryParameterizer,
    TrajectoryUtil,
    TrapezoidProfile,
)

__all__ = [
    "Trajectory",
    "TrajectoryConfig",
    "TrajectoryGenerator",
    "TrajectoryParameterizer",
    "TrajectoryUtil",
    "TrapezoidProfile",
]



import warnings

warnings.warn("wpilib.geometry has moved to wpimath.geometry", FutureWarning)

from wpimath.geometry import Pose2d, Rotation2d, Transform2d, Translation2d, Twist2d

__all__ = ["Pose2d", "Rotation2d", "Transform2d", "Translation2d", "Twist2d"]



import hal
import sys
import traceback
import logging
import wpilib

logger = logging.getLogger("wpilib.ds")


def report_error(
    isError: bool, code: int, error: str, printTrace: bool, exc_info=None
) -> None:
    traceString = ""
    locString = ""

    if printTrace:
        # If an exception is passed in or an exception is present
        if exc_info is None:
            exc_info = sys.exc_info()

        exc = exc_info[0]
        if exc is None:
            tb = traceback.extract_stack()[:-2]
        else:
            tb = traceback.extract_tb(exc_info[2])

        locString = "%s.%s:%s" % (tb[-1][0], tb[-1][1], tb[-1][2])

        trc = "Traceback (most recent call last):\n"
        stackstr = trc + "".join(traceback.format_list(tb))
        if exc is not None:
            stackstr += "  " + ("".join(traceback.format_exception(*exc_info))).lstrip(
                trc
            )
        traceString += "\n" + stackstr

        if exc is None:
            logger.error(error + "\n" + traceString)
        else:
            logger.error(error, exc_info=exc_info)

    elif isError:
        logger.error(error)
    else:
        logger.warning(error)

    hal.sendError(
        isError,
        code,
        False,
        error.encode("utf-8"),
        locString.encode("utf-8"),
        traceString.encode("utf-8"),
        0 if wpilib.RobotBase.isSimulation() else 1,
    )



from . import _init_controller

# autogenerated by 'robotpy-build create-imports wpilib.controller'
from ._controller import (
    HolonomicDriveController,
    PIDController,
    ProfiledPIDController,
    ProfiledPIDControllerRadians,
    RamseteController,
)

__all__ = [
    "HolonomicDriveController",
    "PIDController",
    "ProfiledPIDController",
    "ProfiledPIDControllerRadians",
    "RamseteController",
]

# 2020 compatibility to be removed in 2022
from wpimath.controller import (
    ArmFeedforward,
    ElevatorFeedforward,
    SimpleMotorFeedforward,
    SimpleMotorFeedforwardMeters,
)



# novalidate

import argparse
import inspect
import os
import sys

from os.path import exists
from pkg_resources import iter_entry_points

from .logconfig import configure_logging


def _log_versions():
    import wpilib
    import hal

    import logging

    logger = logging.getLogger("wpilib")

    logger.info("WPILib version %s", wpilib.__version__)
    logger.info("HAL version %s", hal.__version__)

    # should we just die here?
    if hal.__version__.split(".")[:3] != wpilib.__version__.split(".")[:3]:
        logger.warning(
            "Core component versions are not identical! This is not a supported configuration, and you may run into errors!"
        )

    if wpilib.RobotBase.isSimulation():
        logger.info("Running with simulated HAL.")

        # check to see if we're on a RoboRIO
        # NOTE: may have false positives, but it should work well enough
        if exists("/etc/natinst/share/scs_imagemetadata.ini"):
            logger.warning(
                "Running simulation HAL on actual roboRIO! This probably isn't what you want, and will probably cause difficult-to-debug issues!"
            )

    versions = {}

    # Log third party versions
    # -> TODO: in the future, expand 3rd party HAL support here?
    for group in ("robotpylib", "robotpybuild"):
        for entry_point in iter_entry_points(group=group, name=None):
            # Don't actually load the entry points -- just print the
            # packages unless we need to load them
            dist = entry_point.dist
            versions[dist.project_name] = dist.version

    for k, v in versions.items():
        if k not in ("wpilib", "robotpy-hal"):
            logger.info("%s version %s", k, v)


def _enable_faulthandler():
    #
    # In the event of a segfault, faulthandler will dump the currently
    # active stack so you can figure out what went wrong.
    #
    # Additionally, on non-Windows platforms we register a SIGUSR2
    # handler -- if you send the robot process a SIGUSR2, then
    # faulthandler will dump all of your current stacks. This can
    # be really useful for figuring out things like deadlocks.
    #

    import logging

    logger = logging.getLogger("faulthandler")

    try:
        # These should work on all platforms
        import faulthandler

        faulthandler.enable()
    except Exception as e:
        logger.warn("Could not enable faulthandler: %s", e)
        return

    try:
        import signal

        faulthandler.register(signal.SIGUSR2)
        logger.info("registered SIGUSR2 for PID %s", os.getpid())
    except Exception:
        return


class _CustomHelpAction(argparse.Action):
    def __init__(
        self,
        option_strings,
        dest=argparse.SUPPRESS,
        default=argparse.SUPPRESS,
        help=None,
    ):
        super(_CustomHelpAction, self).__init__(
            option_strings=option_strings,
            dest=dest,
            default=default,
            nargs=0,
            help=help,
        )

    def __call__(self, parser, namespace, values, option_string=None):
        parser.print_help()
        parser.exit(1)  # argparse uses an exit code of zero by default


argparse._HelpAction = _CustomHelpAction


def run(robot_class, **kwargs):
    """
    This function gets called in robot.py like so::

        if __name__ == '__main__':
            wpilib.run(MyRobot)

    This function loads available entry points, parses arguments, and
    sets things up specific to RobotPy so that the robot can run. This
    function is used whether the code is running on the roboRIO or
    a simulation.

    :param robot_class: A class that inherits from :class:`.RobotBase`
    :param kwargs: Keyword arguments that will be passed to the executed entry points
    :returns: This function should never return
    """

    # sanity check
    if not hasattr(robot_class, "main"):
        print(
            "ERROR: run() must be passed a robot class that inherits from RobotBase (or IterativeBase/SampleBase)"
        )
        exit(1)

    parser = argparse.ArgumentParser()
    subparser = parser.add_subparsers(dest="command", help="commands")
    subparser.required = True

    parser.add_argument(
        "-v",
        "--verbose",
        action="store_true",
        default=False,
        help="Enable debug logging",
    )

    parser.add_argument(
        "--ignore-plugin-errors",
        action="store_true",
        default=False,
        help="Ignore errors caused by RobotPy plugins (probably should fix or replace instead!)",
    )

    has_cmd = False

    for entry_point in iter_entry_points(group="robotpy", name=None):
        try:
            cmd_class = entry_point.load()
        except ImportError:
            if "--ignore-plugin-errors" in sys.argv:
                print("WARNING: Ignoring error in '%s'" % entry_point)
                continue
            else:
                print(
                    "Plugin error detected in '%s' (use --ignore-plugin-errors to ignore this)"
                    % entry_point
                )
                raise

        cmdparser = subparser.add_parser(
            entry_point.name, help=inspect.getdoc(cmd_class)
        )
        obj = cmd_class(cmdparser)
        cmdparser.set_defaults(cmdobj=obj)
        has_cmd = True

    if not has_cmd:
        parser.error(
            "No entry points defined -- robot code can't do anything. Install packages to add entry points (see README)"
        )
        exit(1)

    options = parser.parse_args()

    configure_logging(options.verbose)

    _log_versions()
    _enable_faulthandler()

    retval = options.cmdobj.run(options, robot_class, **kwargs)

    if retval is None:
        retval = 0
    elif retval is True:
        retval = 0
    elif retval is False:
        retval = 1

    exit(retval)



from . import _init_interfaces

# autogenerated by 'robotpy-build create-imports wpilib.interfaces wpilib.interfaces._wpilibc_interfaces'
from ._interfaces import (
    Accelerometer,
    CounterBase,
    GenericHID,
    Gyro,
    PIDOutput,
    PIDSource,
    PIDSourceType,
    Potentiometer,
    SpeedController,
)

__all__ = [
    "Accelerometer",
    "CounterBase",
    "GenericHID",
    "Gyro",
    "PIDOutput",
    "PIDSource",
    "PIDSourceType",
    "Potentiometer",
    "SpeedController",
]

del _init_interfaces



from . import _init_wpilib

# autogenerated by 'robotpy-build create-imports wpilib wpilib._wpilib'
from ._wpilib import (
    ADXL345_I2C,
    ADXL345_SPI,
    ADXL362,
    ADXRS450_Gyro,
    AddressableLED,
    AnalogAccelerometer,
    AnalogEncoder,
    AnalogGyro,
    AnalogInput,
    AnalogOutput,
    AnalogPotentiometer,
    AnalogTrigger,
    AnalogTriggerOutput,
    AnalogTriggerType,
    BuiltInAccelerometer,
    CAN,
    CANData,
    CANStatus,
    Color,
    Color8Bit,
    Compressor,
    Counter,
    DMC60,
    DigitalGlitchFilter,
    DigitalInput,
    DigitalOutput,
    DigitalSource,
    DoubleSolenoid,
    DriverStation,
    DutyCycle,
    DutyCycleEncoder,
    Encoder,
    Error,
    ErrorBase,
    Field2d,
    FieldObject2d,
    GyroBase,
    I2C,
    InterruptableSensorBase,
    IterativeRobot,
    IterativeRobotBase,
    Jaguar,
    Joystick,
    LiveWindow,
    MotorSafety,
    NidecBrushless,
    Notifier,
    PWM,
    PWMSparkMax,
    PWMSpeedController,
    PWMTalonFX,
    PWMTalonSRX,
    PWMVenom,
    PWMVictorSPX,
    PowerDistributionPanel,
    Preferences,
    Relay,
    RobotBase,
    RobotController,
    RobotState,
    SD540,
    SPI,
    Sendable,
    SendableBase,
    SendableBuilder,
    SendableBuilderImpl,
    SendableChooser,
    SendableChooserBase,
    SendableRegistry,
    SensorUtil,
    SerialPort,
    Servo,
    SlewRateLimiter,
    SmartDashboard,
    Solenoid,
    SolenoidBase,
    Spark,
    SpeedControllerGroup,
    Talon,
    TimedRobot,
    Timer,
    Tracer,
    Ultrasonic,
    Victor,
    VictorSP,
    Watchdog,
    XboxController,
    getCurrentThreadPriority,
    getTime,
    setCurrentThreadPriority,
    wait,
    wpi_error_s_AnalogTriggerLimitOrderError,
    wpi_error_s_AnalogTriggerPulseOutputError,
    wpi_error_s_BadJoystickAxis,
    wpi_error_s_BadJoystickIndex,
    wpi_error_s_CameraServerError,
    wpi_error_s_ChannelIndexOutOfRange,
    wpi_error_s_CommandIllegalUse,
    wpi_error_s_CompassManufacturerError,
    wpi_error_s_CompassTypeError,
    wpi_error_s_CompressorAlreadyDefined,
    wpi_error_s_CompressorNonMatching,
    wpi_error_s_CompressorTaskError,
    wpi_error_s_CompressorUndefined,
    wpi_error_s_DashboardDataCollision,
    wpi_error_s_DashboardDataOverflow,
    wpi_error_s_DriveUninitialized,
    wpi_error_s_DriverStationTaskError,
    wpi_error_s_EnhancedIOMissing,
    wpi_error_s_EnhancedIOPWMPeriodOutOfRange,
    wpi_error_s_IncompatibleMode,
    wpi_error_s_IncompatibleState,
    wpi_error_s_InconsistentArrayValueAdded,
    wpi_error_s_IncorrectBatteryChannel,
    wpi_error_s_InvalidMotorIndex,
    wpi_error_s_InvalidParameter,
    wpi_error_s_JaguarMessageNotFound,
    wpi_error_s_JaguarVersionError,
    wpi_error_s_LineNotOutput,
    wpi_error_s_LoopTimingError,
    wpi_error_s_MismatchedComplexTypeClose,
    wpi_error_s_ModuleIndexOutOfRange,
    wpi_error_s_NetworkTablesBufferFull,
    wpi_error_s_NetworkTablesCorrupt,
    wpi_error_s_NetworkTablesReadError,
    wpi_error_s_NetworkTablesWrongType,
    wpi_error_s_NoAvailableResources,
    wpi_error_s_NonBinaryDigitalValue,
    wpi_error_s_NotAllocated,
    wpi_error_s_NullParameter,
    wpi_error_s_ParameterOutOfRange,
    wpi_error_s_ResourceAlreadyAllocated,
    wpi_error_s_SPIClockRateTooLow,
    wpi_error_s_SPIReadNoData,
    wpi_error_s_SPIReadNoMISO,
    wpi_error_s_SPIWriteNoMOSI,
    wpi_error_s_SampleRateTooHigh,
    wpi_error_s_SmartDashboardMissingKey,
    wpi_error_s_TaskDeletedError,
    wpi_error_s_TaskError,
    wpi_error_s_TaskIDError,
    wpi_error_s_TaskMemoryError,
    wpi_error_s_TaskOptionsError,
    wpi_error_s_TaskPriorityError,
    wpi_error_s_Timeout,
    wpi_error_s_UnsupportedInSimulation,
    wpi_error_s_VoltageOutOfRange,
    wpi_error_value_AnalogTriggerLimitOrderError,
    wpi_error_value_AnalogTriggerPulseOutputError,
    wpi_error_value_BadJoystickAxis,
    wpi_error_value_BadJoystickIndex,
    wpi_error_value_CameraServerError,
    wpi_error_value_ChannelIndexOutOfRange,
    wpi_error_value_CommandIllegalUse,
    wpi_error_value_CompassManufacturerError,
    wpi_error_value_CompassTypeError,
    wpi_error_value_CompressorAlreadyDefined,
    wpi_error_value_CompressorNonMatching,
    wpi_error_value_CompressorTaskError,
    wpi_error_value_CompressorUndefined,
    wpi_error_value_DashboardDataCollision,
    wpi_error_value_DashboardDataOverflow,
    wpi_error_value_DriveUninitialized,
    wpi_error_value_DriverStationTaskError,
    wpi_error_value_EnhancedIOMissing,
    wpi_error_value_EnhancedIOPWMPeriodOutOfRange,
    wpi_error_value_IncompatibleMode,
    wpi_error_value_IncompatibleState,
    wpi_error_value_InconsistentArrayValueAdded,
    wpi_error_value_IncorrectBatteryChannel,
    wpi_error_value_InvalidMotorIndex,
    wpi_error_value_InvalidParameter,
    wpi_error_value_JaguarMessageNotFound,
    wpi_error_value_JaguarVersionError,
    wpi_error_value_LineNotOutput,
    wpi_error_value_LoopTimingError,
    wpi_error_value_MismatchedComplexTypeClose,
    wpi_error_value_ModuleIndexOutOfRange,
    wpi_error_value_NetworkTablesBufferFull,
    wpi_error_value_NetworkTablesCorrupt,
    wpi_error_value_NetworkTablesReadError,
    wpi_error_value_NetworkTablesWrongType,
    wpi_error_value_NoAvailableResources,
    wpi_error_value_NonBinaryDigitalValue,
    wpi_error_value_NotAllocated,
    wpi_error_value_NullParameter,
    wpi_error_value_ParameterOutOfRange,
    wpi_error_value_ResourceAlreadyAllocated,
    wpi_error_value_SPIClockRateTooLow,
    wpi_error_value_SPIReadNoData,
    wpi_error_value_SPIReadNoMISO,
    wpi_error_value_SPIWriteNoMOSI,
    wpi_error_value_SampleRateTooHigh,
    wpi_error_value_SmartDashboardMissingKey,
    wpi_error_value_TaskDeletedError,
    wpi_error_value_TaskError,
    wpi_error_value_TaskIDError,
    wpi_error_value_TaskMemoryError,
    wpi_error_value_TaskOptionsError,
    wpi_error_value_TaskPriorityError,
    wpi_error_value_Timeout,
    wpi_error_value_UnsupportedInSimulation,
    wpi_error_value_VoltageOutOfRange,
)

__all__ = [
    "ADXL345_I2C",
    "ADXL345_SPI",
    "ADXL362",
    "ADXRS450_Gyro",
    "AddressableLED",
    "AnalogAccelerometer",
    "AnalogEncoder",
    "AnalogGyro",
    "AnalogInput",
    "AnalogOutput",
    "AnalogPotentiometer",
    "AnalogTrigger",
    "AnalogTriggerOutput",
    "AnalogTriggerType",
    "BuiltInAccelerometer",
    "CAN",
    "CANData",
    "CANStatus",
    "Color",
    "Color8Bit",
    "Compressor",
    "Counter",
    "DMC60",
    "DigitalGlitchFilter",
    "DigitalInput",
    "DigitalOutput",
    "DigitalSource",
    "DoubleSolenoid",
    "DriverStation",
    "DutyCycle",
    "DutyCycleEncoder",
    "Encoder",
    "Error",
    "ErrorBase",
    "Field2d",
    "FieldObject2d",
    "GyroBase",
    "I2C",
    "InterruptableSensorBase",
    "IterativeRobot",
    "IterativeRobotBase",
    "Jaguar",
    "Joystick",
    "LiveWindow",
    "MotorSafety",
    "NidecBrushless",
    "Notifier",
    "PWM",
    "PWMSparkMax",
    "PWMSpeedController",
    "PWMTalonFX",
    "PWMTalonSRX",
    "PWMVenom",
    "PWMVictorSPX",
    "PowerDistributionPanel",
    "Preferences",
    "Relay",
    "RobotBase",
    "RobotController",
    "RobotState",
    "SD540",
    "SPI",
    "Sendable",
    "SendableBase",
    "SendableBuilder",
    "SendableBuilderImpl",
    "SendableChooser",
    "SendableChooserBase",
    "SendableRegistry",
    "SensorUtil",
    "SerialPort",
    "Servo",
    "SlewRateLimiter",
    "SmartDashboard",
    "Solenoid",
    "SolenoidBase",
    "Spark",
    "SpeedControllerGroup",
    "Talon",
    "TimedRobot",
    "Timer",
    "Tracer",
    "Ultrasonic",
    "Victor",
    "VictorSP",
    "Watchdog",
    "XboxController",
    "getCurrentThreadPriority",
    "getTime",
    "setCurrentThreadPriority",
    "wait",
    "wpi_error_s_AnalogTriggerLimitOrderError",
    "wpi_error_s_AnalogTriggerPulseOutputError",
    "wpi_error_s_BadJoystickAxis",
    "wpi_error_s_BadJoystickIndex",
    "wpi_error_s_CameraServerError",
    "wpi_error_s_ChannelIndexOutOfRange",
    "wpi_error_s_CommandIllegalUse",
    "wpi_error_s_CompassManufacturerError",
    "wpi_error_s_CompassTypeError",
    "wpi_error_s_CompressorAlreadyDefined",
    "wpi_error_s_CompressorNonMatching",
    "wpi_error_s_CompressorTaskError",
    "wpi_error_s_CompressorUndefined",
    "wpi_error_s_DashboardDataCollision",
    "wpi_error_s_DashboardDataOverflow",
    "wpi_error_s_DriveUninitialized",
    "wpi_error_s_DriverStationTaskError",
    "wpi_error_s_EnhancedIOMissing",
    "wpi_error_s_EnhancedIOPWMPeriodOutOfRange",
    "wpi_error_s_IncompatibleMode",
    "wpi_error_s_IncompatibleState",
    "wpi_error_s_InconsistentArrayValueAdded",
    "wpi_error_s_IncorrectBatteryChannel",
    "wpi_error_s_InvalidMotorIndex",
    "wpi_error_s_InvalidParameter",
    "wpi_error_s_JaguarMessageNotFound",
    "wpi_error_s_JaguarVersionError",
    "wpi_error_s_LineNotOutput",
    "wpi_error_s_LoopTimingError",
    "wpi_error_s_MismatchedComplexTypeClose",
    "wpi_error_s_ModuleIndexOutOfRange",
    "wpi_error_s_NetworkTablesBufferFull",
    "wpi_error_s_NetworkTablesCorrupt",
    "wpi_error_s_NetworkTablesReadError",
    "wpi_error_s_NetworkTablesWrongType",
    "wpi_error_s_NoAvailableResources",
    "wpi_error_s_NonBinaryDigitalValue",
    "wpi_error_s_NotAllocated",
    "wpi_error_s_NullParameter",
    "wpi_error_s_ParameterOutOfRange",
    "wpi_error_s_ResourceAlreadyAllocated",
    "wpi_error_s_SPIClockRateTooLow",
    "wpi_error_s_SPIReadNoData",
    "wpi_error_s_SPIReadNoMISO",
    "wpi_error_s_SPIWriteNoMOSI",
    "wpi_error_s_SampleRateTooHigh",
    "wpi_error_s_SmartDashboardMissingKey",
    "wpi_error_s_TaskDeletedError",
    "wpi_error_s_TaskError",
    "wpi_error_s_TaskIDError",
    "wpi_error_s_TaskMemoryError",
    "wpi_error_s_TaskOptionsError",
    "wpi_error_s_TaskPriorityError",
    "wpi_error_s_Timeout",
    "wpi_error_s_UnsupportedInSimulation",
    "wpi_error_s_VoltageOutOfRange",
    "wpi_error_value_AnalogTriggerLimitOrderError",
    "wpi_error_value_AnalogTriggerPulseOutputError",
    "wpi_error_value_BadJoystickAxis",
    "wpi_error_value_BadJoystickIndex",
    "wpi_error_value_CameraServerError",
    "wpi_error_value_ChannelIndexOutOfRange",
    "wpi_error_value_CommandIllegalUse",
    "wpi_error_value_CompassManufacturerError",
    "wpi_error_value_CompassTypeError",
    "wpi_error_value_CompressorAlreadyDefined",
    "wpi_error_value_CompressorNonMatching",
    "wpi_error_value_CompressorTaskError",
    "wpi_error_value_CompressorUndefined",
    "wpi_error_value_DashboardDataCollision",
    "wpi_error_value_DashboardDataOverflow",
    "wpi_error_value_DriveUninitialized",
    "wpi_error_value_DriverStationTaskError",
    "wpi_error_value_EnhancedIOMissing",
    "wpi_error_value_EnhancedIOPWMPeriodOutOfRange",
    "wpi_error_value_IncompatibleMode",
    "wpi_error_value_IncompatibleState",
    "wpi_error_value_InconsistentArrayValueAdded",
    "wpi_error_value_IncorrectBatteryChannel",
    "wpi_error_value_InvalidMotorIndex",
    "wpi_error_value_InvalidParameter",
    "wpi_error_value_JaguarMessageNotFound",
    "wpi_error_value_JaguarVersionError",
    "wpi_error_value_LineNotOutput",
    "wpi_error_value_LoopTimingError",
    "wpi_error_value_MismatchedComplexTypeClose",
    "wpi_error_value_ModuleIndexOutOfRange",
    "wpi_error_value_NetworkTablesBufferFull",
    "wpi_error_value_NetworkTablesCorrupt",
    "wpi_error_value_NetworkTablesReadError",
    "wpi_error_value_NetworkTablesWrongType",
    "wpi_error_value_NoAvailableResources",
    "wpi_error_value_NonBinaryDigitalValue",
    "wpi_error_value_NotAllocated",
    "wpi_error_value_NullParameter",
    "wpi_error_value_ParameterOutOfRange",
    "wpi_error_value_ResourceAlreadyAllocated",
    "wpi_error_value_SPIClockRateTooLow",
    "wpi_error_value_SPIReadNoData",
    "wpi_error_value_SPIReadNoMISO",
    "wpi_error_value_SPIWriteNoMOSI",
    "wpi_error_value_SampleRateTooHigh",
    "wpi_error_value_SmartDashboardMissingKey",
    "wpi_error_value_TaskDeletedError",
    "wpi_error_value_TaskError",
    "wpi_error_value_TaskIDError",
    "wpi_error_value_TaskMemoryError",
    "wpi_error_value_TaskOptionsError",
    "wpi_error_value_TaskPriorityError",
    "wpi_error_value_Timeout",
    "wpi_error_value_UnsupportedInSimulation",
    "wpi_error_value_VoltageOutOfRange",
]


del _init_wpilib

from .cameraserver import CameraServer

try:
    from .version import version as __version__
except ImportError:
    __version__ = "master"

from ._impl.main import run

__all__ += ["CameraServer", "run"]



from . import _init_simulation

# needed for dcmotor return value, TODO fix in robotpy-build
from wpimath._controls._controls import plant as _

# autogenerated by 'robotpy-build create-imports wpilib.simulation'
from ._simulation import (
    ADXRS450_GyroSim,
    AddressableLEDSim,
    AnalogEncoderSim,
    AnalogGyroSim,
    AnalogInputSim,
    AnalogOutputSim,
    AnalogTriggerSim,
    BatterySim,
    BuiltInAccelerometerSim,
    CallbackStore,
    DIOSim,
    DifferentialDrivetrainSim,
    DigitalPWMSim,
    DriverStationSim,
    DutyCycleEncoderSim,
    DutyCycleSim,
    ElevatorSim,
    EncoderSim,
    FlywheelSim,
    GenericHIDSim,
    JoystickSim,
    LinearSystemSim_1_1_1,
    LinearSystemSim_1_1_2,
    LinearSystemSim_2_1_1,
    LinearSystemSim_2_1_2,
    LinearSystemSim_2_2_1,
    LinearSystemSim_2_2_2,
    Mechanism2D,
    PCMSim,
    PDPSim,
    PWMSim,
    RelaySim,
    RoboRioSim,
    SPIAccelerometerSim,
    SimDeviceSim,
    SingleJointedArmSim,
    XboxControllerSim,
    getProgramStarted,
    isTimingPaused,
    pauseTiming,
    restartTiming,
    resumeTiming,
    setProgramStarted,
    setRuntimeType,
    stepTiming,
    stepTimingAsync,
    waitForProgramStart,
)

__all__ = [
    "ADXRS450_GyroSim",
    "AddressableLEDSim",
    "AnalogEncoderSim",
    "AnalogGyroSim",
    "AnalogInputSim",
    "AnalogOutputSim",
    "AnalogTriggerSim",
    "BatterySim",
    "BuiltInAccelerometerSim",
    "CallbackStore",
    "DIOSim",
    "DifferentialDrivetrainSim",
    "DigitalPWMSim",
    "DriverStationSim",
    "DutyCycleEncoderSim",
    "DutyCycleSim",
    "ElevatorSim",
    "EncoderSim",
    "FlywheelSim",
    "GenericHIDSim",
    "JoystickSim",
    "LinearSystemSim_1_1_1",
    "LinearSystemSim_1_1_2",
    "LinearSystemSim_2_1_1",
    "LinearSystemSim_2_1_2",
    "LinearSystemSim_2_2_1",
    "LinearSystemSim_2_2_2",
    "Mechanism2D",
    "PCMSim",
    "PDPSim",
    "PWMSim",
    "RelaySim",
    "RoboRioSim",
    "SPIAccelerometerSim",
    "SimDeviceSim",
    "SingleJointedArmSim",
    "XboxControllerSim",
    "getProgramStarted",
    "isTimingPaused",
    "pauseTiming",
    "restartTiming",
    "resumeTiming",
    "setProgramStarted",
    "setRuntimeType",
    "stepTiming",
    "stepTimingAsync",
    "waitForProgramStart",
]
